@@ -616,16 +616,20 @@ class Variable(Node, Generic[_TypeType, OptionalApplyType]):
         """
         from pytensor.compile.function import function
 
+        ignore_unused_input = kwargs.get("on_unused_input", None) in ("ignore", "warn")
+
         def convert_string_keys_to_variables(inputs_to_values) -> dict["Variable", Any]:
             new_input_to_values = {}
             for key, value in inputs_to_values.items():
                 if isinstance(key, str):
                     matching_vars = get_var_by_name([self], key)
                     if not matching_vars:
-                        raise Exception(f"{key} not found in graph")
+                        if not ignore_unused_input:
+                            raise ValueError(f"{key} not found in graph")
                     elif len(matching_vars) > 1:
-                        raise Exception(f"Found multiple variables with name {key}")
-                    new_input_to_values[matching_vars[0]] = value
+                        raise ValueError(f"Found multiple variables with name {key}")
+                    else:
+                        new_input_to_values[matching_vars[0]] = value
                 else:
                     new_input_to_values[key] = value
             return new_input_to_values
